/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.Collections;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// This is the main class of the application. This class will be used by the application to:
    /// 1) Manipulate the layout manager that will be used to layout the dependency diagrams. Only a single
    /// layout manager can be active at any given time.
    /// 2) Manipulate the renderer(s) used to render the dependency diagrams. More than one renderer can be used.
    /// 3) Provide a dependency and have it layed out into a diagram.
    /// </summary>
    public class DependencyRenderer
    {

        /// <summary>
        /// The shape factory used by this renderer to produce the shapes used in the rendering process. This
        /// variable is set in the constructor, is immutable (the reference only) once set and will be used in
        /// the render method. This variable has been marked readonly to enforce it's immutablility (again -
        /// reference only).
        /// </summary>
        private readonly ShapeFactory shapeFactory;

        /// <summary>
        /// The variable represents the renderes that will be used to render the diagram. Each element in
        /// the list will be of DiagramRenderer type. This value is set during class construction time,
        /// is immutable (the reference not the contents) and will be used in the render method. The list
        /// content can be accessed via the addDiagramRenderer(), removeDiagramRenderer() and DiagramRenderers
        /// property. This variable has been marked readonly to ensure it's immutability (again reference only).
        /// </summary>
        private readonly IList renderers = new ArrayList();

        /// <summary>
        /// The variable represents the layout manager to use when laying the dependency diagram out. This variable
        /// will initially be set at object construction time, is mutatable by the setter method and will be
        /// accessed in both the getter method and the render method.
        /// </summary>
        private ILayoutManager layoutManager = new FixedGridLayoutManager();

        /// <summary>
        /// This is the property for the layoutManager variable. This is a simple setter/getter for the
        /// layoutManager variable.
        /// </summary>
        /// <exception cref="ArgumentNullException">if value is null</exception>
        public ILayoutManager LayoutManager
        {
            get
            {
                return this.layoutManager;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("layoutManager");
                }
                this.layoutManager = value;
            }
        }

        /// <summary>
        /// This is the property for the renderers variable. This property defines ONLY the getter (no setter).
        /// This property getter will return a ReadOnly version of the renderers.
        /// </summary>
        public IList DiagramRenderers
        {
            get
            {
                return ArrayList.ReadOnly(this.renderers);
            }
        }

        /// <summary>
        /// Constructs the DependencyRenderer with a default ShapeFactory. This method will simply call
        /// 'this(new ShapeFactory())'.
        /// </summary>
        public DependencyRenderer() : this(new ShapeFactory())
        {
        }

        /// <summary>
        /// Constructs the DependencyRenderer from the specified shapeFactory. This method will simply save
        /// the shapeFactory reference to the like named variable.
        /// </summary>
        /// <param name="shapeFactory">A non-null shapeFactory to use.</param>
        /// <exception cref="ArgumentNullException">if shapeFactory is null</exception>
        public DependencyRenderer(ShapeFactory shapeFactory)
        {
            if (shapeFactory == null)
            {
                throw new ArgumentNullException("shapeFactory");
            }
            this.shapeFactory = shapeFactory;
        }

        /// <summary>
        /// Adds a new renderer to the list of renderers that will be used to render a dependency diagram.
        /// This method simply adds the renderer to the renderers list.
        /// </summary>
        /// <param name="renderer">the renderer to add to the list of renderers</param>
        /// <exception cref="ArgumentNullException">if renderer is null</exception>
        public void AddDiagramRenderer(IDiagramRenderer renderer)
        {
            if (renderer == null)
            {
                throw new ArgumentNullException("renderer");
            }
            this.renderers.Add(renderer);
        }

        /// <summary>
        /// This method will remove the specified renderer from the list of renderers.
        /// </summary>
        /// <param name="renderer">The renderer to remove</param>
        /// <exception cref="ArgumentNullException">if renderer is null</exception>
        public void RemoveDiagramRenderer(IDiagramRenderer renderer)
        {
            if (renderer == null)
            {
                throw new ArgumentNullException("renderer");
            }
            this.renderers.Remove(renderer);
        }

        /// <summary>
        /// This method is the method that will be used by the application to render the dependency diagram.
        /// The layout manager will be used to layout the diagram against each renderer (note: because of
        /// differences in the font metrics between renderers, the layout must be repeated for each renderer
        /// to account for those differences). This method will:
        /// 1) Iterate over the renderers and for each renderer ...
        /// 2) Call renderer.GetGraphics(dependency) to get a IGraphics object for that renderer.
        /// If an RendererException is thrown, simply let it pass through to the caller.
        /// 3) Call the layoutManager.Layout(dependency, shapeFactory, graphics) to layout the diagram.
        /// If an RendererException is thrown, do step #4 and then throw to the caller.
        /// 4) Call graphics.Complete() to allow the IGraphics implementation to save resources.
        /// If a RendererException is thrown, let it pass through to the caller.
        /// 5) Wrap steps #3 and #4 in a finally block to ensure that the graphics.Dispose() is called
        /// (to dispose of graphics resources).
        /// 6) Repeat until all renderers are done
        /// </summary>
        /// <param name="dependency">The dependency that will be rendered</param>
        /// <exception cref="ArgumentNullException">if dependency is null</exception>
        /// <exception cref="RendererException">if an exception occurs during rendering</exception>
        public void Render(Dependency dependency)
        {
            if (dependency == null)
            {
                throw new ArgumentNullException("dependency");
            }
            foreach (IDiagramRenderer renderer in this.renderers)
            {
                IGraphics graphics = renderer.GetGraphics(dependency);
                try
                {
                    layoutManager.Layout(dependency, shapeFactory, graphics);
                    graphics.Complete();
                }
                finally
                {
                    graphics.Dispose();
                }
            }
        }

    }

}
